import os
import sys
import time

try:
    import libximc.highlevel as ximc
except ImportError:
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    import libximc.highlevel as ximc


def test_status(axis: ximc.Axis) -> None:
    print("\nStatus:")
    status = axis.get_status()
    print(" rpm: {},".format(status.CurSpeed), end="")
    print(" pos: {},".format(status.CurPosition), end="")
    print(" upwr: {},".format(status.Upwr), end="")
    print(" ipwr: {},".format(status.Ipwr), end="")
    print(" flags: {},".format(status.Flags), end="")
    print(" mvsts: {}".format(status.MvCmdSts))


def test_get_position(axis: ximc.Axis) -> 'tuple':
    print("\nRead position")
    pos = axis.get_position()
    print("Position: {} steps, {} microsteps".format(pos.Position, pos.uPosition))
    return pos.Position, pos.uPosition


def test_left(axis: ximc.Axis) -> None:
    print("\nMoving left for 3 seconds...")
    axis.command_left()
    time.sleep(3)


def test_move(axis: ximc.Axis, distance: int, udistance: int) -> None:
    print("\nGoing to {} steps, {} microsteps".format(distance, udistance))
    axis.command_move(distance, udistance)


def test_wait_for_stop(axis: ximc.Axis, interval: int) -> None:
    print("\nWaiting for stop...")
    axis.command_wait_for_stop(interval)


def test_get_speed(axis: ximc.Axis) -> int:
    print("\nGet speed")
    move_settings = axis.get_move_settings()
    print("Speed: {}".format(move_settings.Speed))
    return move_settings.Speed


def test_set_speed(axis: ximc.Axis, speed: int) -> None:
    print("\nSet new speed {}".format(speed))
    move_settings = axis.get_move_settings()
    move_settings.Speed = speed
    axis.set_move_settings(move_settings)


def test_set_microstep_mode_256(axis: ximc.Axis) -> None:
    print("\nSet microstep mode to 256")
    engine_settings = axis.get_engine_settings()

    # Change MicrostepMode parameter to MICROSTEP_MODE_FRAC_256
    # (use MICROSTEP_MODE_FRAC_128, MICROSTEP_MODE_FRAC_64 ... for other microstep modes)
    engine_settings.MicrostepMode = ximc.MicrostepMode.MICROSTEP_MODE_FRAC_256

    axis.set_engine_settings(engine_settings)


def choose_device() -> str:
    devenum = search_devices()
    if len(devenum) < 1:
        print("The real controller is not found or busy with another app.\nThe virtual controller is opening to check "
              "the operation of the library...")
        device_name = create_virtual_device_name()
    else:
        device_name = get_device_name_from_list(devenum)
    return device_name


def create_virtual_device_name() -> str:
    path = os.path.join(os.path.expanduser("~"), "virt_controller.bin")
    device_name = "xi-emu:///" + path
    print("Virtual controller file path: '{}'".format(path))
    return device_name


def get_device_name_from_list(devenum) -> str:
    while True:
        print_device_list(devenum)
        print("When using an Ethernet (TCP protocol) connection, multi-user interaction with the device is possible. "
              "In this case, the same device may be controlled simultaneously from multiple sources, which can "
              "potentially lead to control conflicts.\nEnter device number from the list:")
        try:
            number = int(input())
        except Exception:
            continue

        if 1 <= number <= len(devenum):
            device_name = devenum[number - 1]["uri"]
            print("Selected device: '{}'".format(device_name))
            return device_name

        if number == len(devenum) + 1:
            return create_virtual_device_name()


def print_device_list(devenum) -> None:
    print("\nFound devices list:")
    for i, device in enumerate(devenum, start=1):
        print(" #{}. {}".format(i, device["uri"]))
    print(" #{}. virtual".format(len(devenum) + 1))


def search_devices():
    print("\nSearching for devices...")
    probe_flags = ximc.EnumerateFlags.ENUMERATE_PROBE | ximc.EnumerateFlags.ENUMERATE_NETWORK
    enumerate_hints = "addr="
    devenum = ximc.enumerate_devices(probe_flags, enumerate_hints)
    if not devenum:
        print("No devices found")
    else:
        print("Found {} devices".format(len(devenum) + 1))
    return devenum


def warn_user_about_modifications() -> bool:
    while True:
        print("\nThe script will modify the following controller settings:\n"
              " engine settings\n"
              " move settings\nContinue? [y/n]:")
        answer = input()
        if answer.lower() == "y":
            return False

        if answer.lower() == "n":
            return True


def main() -> None:
    print("This is a ximc test program\nlibximc version " + ximc.ximc_version())

    if warn_user_about_modifications():
        return

    device_name = choose_device()
    print("\nOpening device...", end="")
    try:
        axis = ximc.Axis(device_name)
        axis.open_device()
    except Exception:
        print("\nError opening device\nPress Enter to exit...")
        return
    print(" Done")


    # ******************************************* #
    #                    Tests                    #
    # ******************************************* #
    test_status(axis)
    test_set_microstep_mode_256(axis)
    start_pos, ustart_pos = test_get_position(axis)
    # First move
    test_left(axis)
    test_get_position(axis)
    # Second move
    current_speed = test_get_speed(axis)
    test_set_speed(axis, current_speed // 2)
    test_move(axis, start_pos, ustart_pos)
    test_wait_for_stop(axis, 100)
    test_status(axis)

    print("\nClosing...", end=" ")
    axis.close_device()
    print("Done")
    input("Press Enter to close the script...")


if __name__ == "__main__":
    main()



from enum import Enum
from support_routines import interface_repeater
try:
    import libximc.highlevel as ximc
except ImportError:
    import sys
    import os
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    import libximc.highlevel as ximc
    print("Success!")


class SettingsManager:
    class Action(Enum):
        QUIT = "q"
        MOVE_SETTINGS = "m"
        MOVE_SETTINGS_CALB = "c"
        FEEDBACK_SETTINGS = "f"
        EDGES_SETTINGS = "e"
        MICROSTEP_SETTINGS = "s"
        USER_UNIT_SETTINGS = "u"
        LOAD_CORRECTION_TABLE = "l"

    def __init__(self, axis: ximc.Axis):
        self.axis = axis
        self.feedback_settings_manager = FeedbackSettings(axis)

    def start(self) -> None:
        """
        Manager of the controller settings.

        This function, among other settings, allows you to load the coordinate correction table.
        Follow the on-screen instructions to change the settings.

        :param lib: structure for accessing the functionality of the libximc library.
        :param device_id: device id.

        note:
            The device_id parameter in this function is a C pointer, unlike most library functions that use this
            parameter
        """
        action = self.ask_for_action()
        while action != SettingsManager.Action.QUIT:
            if action == SettingsManager.Action.MOVE_SETTINGS:
                self.change_move_settings()
            if action == SettingsManager.Action.MOVE_SETTINGS_CALB:
                self.change_move_settings_calb()
            if action == SettingsManager.Action.FEEDBACK_SETTINGS:
                self.feedback_settings_manager.start()
            if action == SettingsManager.Action.EDGES_SETTINGS:
                self.change_edges_settings()
            if action == SettingsManager.Action.MICROSTEP_SETTINGS:
                self.change_microstep_mode()
            if action == SettingsManager.Action.USER_UNIT_SETTINGS:
                self.change_user_unit_mode()
            if action == SettingsManager.Action.LOAD_CORRECTION_TABLE:
                print("Correction table loading...\n"
                      "You can use a relative or absolute file path.")
                path = input("Enter the path: ")
                self.axis.set_correction_table(path)
            action = self.ask_for_action()

    @interface_repeater
    def ask_for_action(self) -> Action:
        print("\n===== Settings menu =====\n"
              "Select a group of settings:\n"
              "Q or q keys\t-\treturn to the main menu\n"
              "M or m keys\t-\tmovement settings\n"
              "C or c keys\t-\tmovement settings (calb)\n"
              "F or f keys\t-\tfeedback settings\n"
              "E or e keys\t-\tedges settings\n"
              "S or s keys\t-\tmicro step mode settings\n"
              "U or u keys\t-\tuser unit settings\n"
              "L or l keys\t-\tload correction table\n")
        return SettingsManager.Action(input("Your choice: ").lower())

    def change_move_settings(self) -> None:
        """Set speed, acceleration, and deceleration"""
        print("\nGet motion settings")
        move_settings = self.axis.get_move_settings()
        print("Current speed: {}".format(move_settings.Speed))
        print("Current acceleration: {}".format(move_settings.Accel))
        print("Current deceleration: {}".format(move_settings.Decel) + "\n")

        new_speed = interface_repeater(lambda: int(input("Input speed: ")))()
        new_asel = interface_repeater(lambda: int(input("Input acceleration: ")))()
        new_decel = interface_repeater(lambda: int(input("Input deceleration: ")))()

        move_settings.Speed = new_speed
        move_settings.Accel = new_asel
        move_settings.Decel = new_decel

        self.axis.set_move_settings(move_settings)

    def change_move_settings_calb(self) -> None:
        """Set speed, acceleration, and deceleration"""
        print("\nGet motion settings")
        move_settings = self.axis.get_move_settings_calb()
        print("Current speed: {}".format(move_settings.Speed))
        print("Current acceleration: {}".format(move_settings.Accel))
        print("Current deceleration: {}".format(move_settings.Decel) + "\n")

        new_speed = interface_repeater(lambda: float(input("Input speed: ")))()
        new_asel = interface_repeater(lambda: float(input("Input acceleration: ")))()
        new_decel = interface_repeater(lambda: float(input("Input deceleration: ")))()

        move_settings.Speed = new_speed
        move_settings.Accel = new_asel
        move_settings.Decel = new_decel

        self.axis.set_move_settings_calb(move_settings)

    def change_edges_settings(self) -> None:
        """View and configure the limit switch mode."""
        # Get current feedback settings from controller
        edges_settings = self.axis.get_edges_settings()

        print("Current BorderFlags: {}".format(edges_settings.BorderFlags))
        if (edges_settings.BorderFlags & ximc.BorderFlags.BORDER_IS_ENCODER):
            print("\tBORDER_IS_ENCODER is set: The borders are fixed by predetermined encoder values.")
        else:
            print("\tBORDER_IS_ENCODER is unset: The borders are placed on limit switches.")

        if (edges_settings.BorderFlags & ximc.BorderFlags.BORDER_STOP_LEFT):
            print("\tBORDER_STOP_LEFT is set: The motor stops when it reaches the left border.")
        else:
            print("\tBORDER_STOP_LEFT is unset: The motor continues to move when it reaches the left border.")

        if (edges_settings.BorderFlags & ximc.BorderFlags.BORDER_STOP_RIGHT):
            print("\tBORDER_STOP_RIGHT is set: The motor stops when it reaches the right border.")
        else:
            print("\tBORDER_STOP_RIGHT is unset: The motor continues to move when it reaches the right border.")

        if (edges_settings.BorderFlags & ximc.BorderFlags.BORDERS_SWAP_MISSET_DETECTION):
            print("\tBORDERS_SWAP_MISSET_DETECTION is set: Detection of incorrect setting of limit switches is "
                  "enabled. The motor will stop on both borders.")
        else:
            print("\tBORDERS_SWAP_MISSET_DETECTION is unset: Detection of incorrect setting of limit switches is "
                  "disabled.")

        if (edges_settings.EnderFlags & ximc.EnderFlags.ENDER_SWAP):
            print("\tENDER_SWAP is set: The first limit switch is on the right side.")
        else:
            print("\tENDER_SWAP is unset: The first limit switch is on the right side.")

        if (edges_settings.EnderFlags & ximc.EnderFlags.ENDER_SW1_ACTIVE_LOW):
            print("\tENDER_SW1_ACTIVE_LOW is set: Limit switch connected to pin SW1 is triggered by a low level on "
                  "pin.")
        else:
            print("\tENDER_SW1_ACTIVE_LOW is unset:Limit switch connected to pin SW1 is triggered by a high level on "
                  "pin.")

        if (edges_settings.EnderFlags & ximc.EnderFlags.ENDER_SW2_ACTIVE_LOW):
            print("\tENDER_SW2_ACTIVE_LOW is set: Limit switch connected to pin SW2 is triggered by a low level on "
                  "pin.")
        else:
            print("\tENDER_SW2_ACTIVE_LOW is unset: Limit switch connected to pin SW2 is triggered by a high level on "
                  "pin.")

        # The position of the boundaries.
        print("The positions of the borders")
        print("Coordinate of the left border: Pos {0}, uPos {1}".format(edges_settings.LeftBorder,
                                                                        edges_settings.uLeftBorder))
        print("Coordinate of the right border: Pos {0}, uPos {1} \n".format(edges_settings.RightBorder,
                                                                            edges_settings.uRightBorder))

        # Enter the values for the flags.
        edges_settings.BorderFlags = self.ask_for_border_flags()
        edges_settings.EnderFlags = self.ask_for_ender_flags()

        # Enter borders.
        key_press = input("Do you want to reset the borders? Y/N ")
        if key_press == "Y" or key_press == "y":
            try:
                edges_settings.LeftBorder = interface_repeater(lambda: int(input("Enter the left border: ")))()
                edges_settings.RightBorder = interface_repeater(lambda: int(input("Enter the right border: ")))()
            except Exception:
                print("Left border {0}, right border {1}".format(edges_settings.LeftBorder, edges_settings.RightBorder))
        self.axis.set_edges_settings(edges_settings)

    def ask_for_border_flags(self) -> None:
        res = ximc.BorderFlags(0)
        print("New settings:")
        for flag in ximc.BorderFlags:
            print("Set {}? Y/N".format(flag.name), end=" ")
            key_pressed = input()
            res |= (flag if key_pressed == "Y" or key_pressed == "y" else res)
        return res

    def ask_for_ender_flags(self) -> None:
        res = ximc.EnderFlags(0)
        for flag in ximc.EnderFlags:
            print("Set {}? Y/N".format(flag.name), end=" ")
            key_pressed = input()
            res |= (flag if key_pressed == "Y" or key_pressed == "y" else res)
        return res

    def change_microstep_mode(self) -> None:
        """Setting the microstep mode. Works only with stepper motors"""
        print("\nMicrostep mode settings. This setting is only available for stepper motors.")
        # Get current engine settings from controller
        engine_settings = self.axis.get_engine_settings()
        print("Current mode: {}".format(engine_settings.MicrostepMode))
        engine_settings.MicrostepMode = self.get_microstep_mode()
        self.axis.set_engine_settings(engine_settings)
        print("The mode is set to",  engine_settings.MicrostepMode)

    @interface_repeater
    def get_microstep_mode(self) -> ximc.MicrostepMode:
        print("Select microstep mode:")
        modes = [mode for mode in ximc.MicrostepMode]
        for i, mode in enumerate(modes, 1):
            print("To set mode {0} - enter {1}".format(mode, i))
        choosen_index = int(input()) - 1
        if choosen_index < 0 or choosen_index >= len(modes):
            raise ValueError("Wrong index!")
        return modes[choosen_index]

    def change_user_unit_mode(self) -> None:
        """User unit mode settings"""
        print("\nUser unit mode settings.")
        print("Current user unit coordinate multiplier = {0} \n".format(self.axis.get_calb()[0]))
        engine_settings = self.axis.get_engine_settings()
        self.axis.set_calb(interface_repeater(lambda: float(input("Set new coordinate multiplier: ")))(),
                           engine_settings.MicrostepMode)


class FeedbackSettings:
    def __init__(self, axis: ximc.Axis):
        self.axis = axis

    def start(self) -> None:
        # Get current feedback settings from controller
        feedback_settings = self.axis.get_feedback_settings()

        print("Feedback type: {}".format(feedback_settings.FeedbackType))
        if feedback_settings.FeedbackFlags & ximc.FeedbackFlags.FEEDBACK_ENC_REVERSE:
            print("Encoder mode: ENC_REVERSE")
        else:
            print("Encoder mode: ENC_NO_REVERSE")

        if feedback_settings.FeedbackFlags & ximc.FeedbackFlags.FEEDBACK_ENC_ADAPTIVE_HOLDING:
            print("Adaptive holding is enabled")
        else:
            print("Adaptive holding is disabled")

        if feedback_settings.FeedbackFlags & ximc.FeedbackFlags.FEEDBACK_ENC_TYPE_SINGLE_ENDED:
            print("Encoder type: FEEDBACK_ENC_TYPE_SINGLE_ENDED")
        if feedback_settings.FeedbackFlags & ximc.FeedbackFlags.FEEDBACK_ENC_TYPE_DIFFERENTIAL:
            print("Encoder type: FEEDBACK_ENC_TYPE_DIFFERENTIAL")

        # Select a new feedback mode
        print("NOTE: in case of Encoder or Encoder-Mediated type, controller uses rpm instead of steps/sec.")
        feedback_settings.FeedbackType = self.ask_for_feedback_type()
        feedback_settings.FeedbackFlags = self.ask_for_feedback_flag()
        self.axis.set_feedback_settings(feedback_settings)

    @interface_repeater
    def ask_for_feedback_type(self) -> ximc.FeedbackType:
        types = [t for t in ximc.FeedbackType]
        print("Select a new feedback type:")
        for i, t in enumerate(types, 1):
            print("To set {0} - enter {1}".format(t.name, i))
        choosen_index = int(input("\nYour choice: ")) - 1
        if choosen_index < 0 or choosen_index >= len(types):
            raise ValueError("Wrong index!")
        return types[choosen_index]

    @interface_repeater
    def ask_for_feedback_flag(self) -> ximc.FeedbackFlags:
        flags = (ximc.FeedbackFlags.FEEDBACK_ENC_TYPE_AUTO,
                 ximc.FeedbackFlags.FEEDBACK_ENC_TYPE_DIFFERENTIAL,
                 ximc.FeedbackFlags.FEEDBACK_ENC_TYPE_SINGLE_ENDED)

        print("Select a new feedback flag:")
        for i, f in enumerate(flags, 1):
            print("To set {0} - enter {1}".format(f.name, i))
        choosen_index = int(input("\nYour choice: ")) - 1
        if choosen_index < 0 or choosen_index >= len(flags):
            raise ValueError("Wrong index!")

        key = input("Do you want to enable reverse? Y/N ")
        if key == "Y" or key == "y":
            enc_reverse_enabled = True
        else:
            enc_reverse_enabled = False
            return flags[choosen_index]

        key = input("Do you want to adaptive holding? Y/N ")
        if key == "Y" or key == "y":
            adaptive_holding_enabled = True
        else:
            adaptive_holding_enabled = False

        if enc_reverse_enabled and adaptive_holding_enabled:
            return flags[choosen_index] | ximc.FeedbackFlags.FEEDBACK_ENC_REVERSE | \
                ximc.FeedbackFlags.FEEDBACK_ENC_ADAPTIVE_HOLDING
        elif enc_reverse_enabled:
            return flags[choosen_index] | ximc.FeedbackFlags.FEEDBACK_ENC_REVERSE
        elif adaptive_holding_enabled:
            return flags[choosen_index] | ximc.FeedbackFlags.FEEDBACK_ENC_ADAPTIVE_HOLDING
        else:
            return flags[choosen_index]



from abc import ABC
from enum import Enum
from time import sleep
from support_routines import interface_repeater
try:
    import libximc.highlevel as ximc
except ImportError:
    import sys
    import os
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    import libximc.highlevel as ximc
    print("Success!")


class MovementManagerBase(ABC):
    class Action(Enum):
        QUIT = "q"
        LEFT = "l"
        RIGHT = "r"
        MOVE = "m"
        SHIFT = "s"
        HOME = "h"
        ZERO = "z"

    def __init__(self, axis: ximc.Axis):
        self.axis = axis

    def start(self):
        action = self.ask_for_action()
        while action != MovementManager.Action.QUIT:
            if action == MovementManager.Action.MOVE:
                print("Moving to position...")
                self.move(*self.ask_for_position())
                self.verbose_wait_for_stop(100)

            elif action == MovementManager.Action.SHIFT:
                print("Shifting on position delta...")
                self.movr(*self.get_delta_position())
                self.verbose_wait_for_stop(100)

            elif action == MovementManager.Action.LEFT:
                self.axis.command_left()
                print("Moving to the left... Press Enter to stop.")
                input()
                self.axis.command_sstp()

            elif action == MovementManager.Action.RIGHT:
                self.axis.command_right()
                print("Moving to the right... Press Enter to stop.")
                input()
                self.axis.command_sstp()

            elif action == MovementManager.Action.HOME:
                print("Homing...")
                self.axis.command_home()
                self.verbose_wait_for_stop(100)

            elif action == MovementManager.Action.ZERO:
                print("Zeroing...")
                self.axis.command_zero()

            print("Waiting for movement completion...")
            self.axis.command_wait_for_stop(100)
            action = self.ask_for_action()

    @interface_repeater
    def ask_for_action(self) -> Action:
        print("\n===== Movement menu =====\n"
              "Choose movement:\n"
              "Q or q keys\t-\treturn to the main menu\n"
              "L or l keys\t-\tmove to the left\n"
              "R or r keys\t-\tmove to the right. Press and hold the key\n"
              "M or m keys\t-\tmove to position(move)\n"
              "S or s keys\t-\tposition shift(movr)\n"
              "H or h keys\t-\tHOME position\n"
              "Z or z keys\t-\tZERO position\n")
        key = input("Your choice: ")
        return MovementManagerBase.Action(key.lower())

    def move(self, *args) -> None:
        pass

    def movr(self, *args) -> None:
        pass

    def ask_for_position(self):
        pass

    def get_delta_position(self):
        pass

    def verbose_wait_for_stop(self, refresh_interval_ms):
        pass


class MovementManager(MovementManagerBase):
    @interface_repeater
    def ask_for_position(self) -> "tuple":
        print("Enter target position")
        print("An integer part (in steps): ", end="")
        position = int(input())
        print("A fractional part (in micro steps): ", end="")
        uposition = int(input())
        return position, uposition

    @interface_repeater
    def get_delta_position(self) -> "tuple":
        print("Enter position delta")
        print("An integer part (in steps): ", end="")
        delta = int(input())
        print("A fractional part (in micro steps): ", end="")
        udelta = int(input())
        return delta, udelta

    def move(self, *args) -> None:
        self.axis.command_move(*args)

    def movr(self, *args) -> None:
        self.axis.command_movr(*args)

    def verbose_wait_for_stop(self, refresh_interval_ms: int) -> None:
        """This function performs dynamic output coordinate in the process of moving."""
        while self.axis.get_status().MvCmdSts & ximc.MvcmdStatus.MVCMD_RUNNING:
            position = self.axis.get_position()
            print("Position: {} steps {} microsteps".format(position.Position, position.uPosition))
            sleep(refresh_interval_ms / 1000)
        position = self.axis.get_position()
        print("Position: {} steps {} microsteps".format(position.Position, position.uPosition))


class MovementManagerCalb(MovementManagerBase):
    @interface_repeater
    def ask_for_position(self) -> float:
        print("Enter target position.")
        position = float(input("Target position: "))
        return (position, )

    @interface_repeater
    def get_delta_position(self) -> float:
        print("Enter position delta.")
        delta = float(input("Position delta: "))
        return (delta, )

    def move(self, *args) -> None:
        self.axis.command_move_calb(*args)

    def movr(self, *args) -> None:
        self.axis.command_movr_calb(*args)

    def verbose_wait_for_stop(self, refresh_interval_ms: int) -> None:
        """This function performs dynamic output coordinate in the process of moving."""
        while self.axis.get_status().MvCmdSts & ximc.MvcmdStatus.MVCMD_RUNNING:
            position = self.axis.get_position_calb()
            print("Position: {}".format(position.Position))
            sleep(refresh_interval_ms / 1000)
        position = self.axis.get_position_calb()
        print("Position: {}".format(position.Position))



from enum import Enum
from support_routines import interface_repeater
try:
    import libximc.highlevel as ximc
except ImportError:
    import sys
    import os
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    import libximc.highlevel as ximc
    print("Success!")


class EXTIOManager:
    """External input / output settings Manager."""
    class InputFlag(Enum):
        EXTIO_SETUP_MODE_IN_STOP = 1
        EXTIO_SETUP_MODE_IN_PWOF = 2
        EXTIO_SETUP_MODE_IN_MOVR = 3
        EXTIO_SETUP_MODE_IN_HOME = 4
        EXTIO_SETUP_MODE_IN_ALARM = 5

    class OutputFlag(Enum):
        EXTIO_SETUP_MODE_OUT_MOVING = 1
        EXTIO_SETUP_MODE_OUT_ALARM = 2
        EXTIO_SETUP_MODE_OUT_MOTOR_ON = 3

    class Direction(Enum):
        INPUT = "i"
        OUTPUT = "o"
        INV_OUTPUT = "r"

    def __init__(self, axis: ximc.Axis):
        self.axis = axis

    def start(self) -> None:
        direction = self.get_direction()
        extio_settings = self.axis.get_extio_settings()
        if direction == EXTIOManager.Direction.INPUT:
            extio_settings.EXTIOSetupFlags = 0  # Set input direction

            flag = self.get_input_flag()
            if flag == EXTIOManager.InputFlag.EXTIO_SETUP_MODE_IN_STOP:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_IN_STOP
                print("The device will stop moving on external input.")
            elif flag == EXTIOManager.InputFlag.EXTIO_SETUP_MODE_IN_PWOF:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_IN_PWOF
                print("The device will perform power OFF on external input.")
            elif flag == EXTIOManager.InputFlag.EXTIO_SETUP_MODE_IN_MOVR:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_IN_MOVR
                print("The device will launch movr command with previous settings on external input.")
            elif flag == EXTIOManager.InputFlag.EXTIO_SETUP_MODE_IN_HOME:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_IN_HOME
                print("The device will perform homing on external input.")
            elif flag == EXTIOManager.InputFlag.EXTIO_SETUP_MODE_IN_ALARM:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_IN_ALARM
                print("The device will enter an alarm state on external input.")
        elif direction == EXTIOManager.Direction.OUTPUT or direction == EXTIOManager.Direction.INV_OUTPUT:
            extio_settings.EXTIOSetupFlags = ximc.ExtioSetupFlags.EXTIO_SETUP_OUTPUT
            if direction == EXTIOManager.Direction.INV_OUTPUT:
                extio_settings.EXTIOSetupFlags |= ximc.ExtioSetupFlags.EXTIO_SETUP_INVERT

            flag = self.get_output_flag()
            if flag == EXTIOManager.OutputFlag.EXTIO_SETUP_MODE_OUT_MOVING:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_OUT_MOVING
                print("EXTIO output will stay active during moving.")
            elif flag == EXTIOManager.OutputFlag.EXTIO_SETUP_MODE_OUT_ALARM:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_OUT_ALARM
                print("EXTIO output will stay active during alarm state")
            elif flag == EXTIOManager.OutputFlag.EXTIO_SETUP_MODE_OUT_MOTOR_ON:
                extio_settings.EXTIOModeFlags = ximc.ExtioModeFlags.EXTIO_SETUP_MODE_OUT_MOTOR_ON
                print("EXTIO output will stay active during the windings are powered")
        else:
            raise RuntimeError("Wrong direction! Got {}".format(direction))
        self.axis.set_extio_settings(extio_settings)

    @interface_repeater
    def get_input_flag(self) -> InputFlag:
        print("Set EXTIO input mode.\n"
              "Choose flag:\n"
              "1 - EXTIO_SETUP_MODE_IN_STOP\n"
              "2 - EXTIO_SETUP_MODE_IN_PWOF\n"
              "3 - EXTIO_SETUP_MODE_IN_MOVR\n"
              "4 - EXTIO_SETUP_MODE_IN_HOME\n"
              "5 - EXTIO_SETUP_MODE_IN_ALARM\n")
        return EXTIOManager.InputFlag(int(input("Your choice: ")))

    @interface_repeater
    def get_output_flag(self) -> OutputFlag:
        print("Set EXTIO output mode.\n"
              "Choose flag:\n"
              "1 - EXTIO_SETUP_MODE_OUT_MOVING\n"
              "2 - EXTIO_SETUP_MODE_OUT_ALARM\n"
              "3 - EXTIO_SETUP_MODE_OUT_MOTOR_ON\n")
        return EXTIOManager.OutputFlag(int(input("Your choice: ")))

    @interface_repeater
    def get_direction(self) -> Direction:
        print("Use output as input or output?\n"
              "Choose mode:\n"
              "I or i keys - input\n"
              "O or o keys - output\n"
              "R or r keys - inverted output\n")
        return EXTIOManager.Direction(input("Your choice: ").lower())



"""This module is an extended example of using the libximc library to control 8SMC series using the Python language"""
import os
import platform
from enum import Enum
from support_routines import interface_repeater
from EXTIO_Manager.extio_manager import EXTIOManager
from Movement_Manager.movement_manager import MovementManager, MovementManagerCalb
from Settings_Manager.settings_manager import SettingsManager
try:
    import libximc.highlevel as ximc
    print("Use libximc {} that has been found among the pip installed packages".format(ximc.ximc_version()))
except ImportError:
    print("Warning! libximc cannot be found among the pip installed packages. Did you forget to install it via pip?\n"
          "Trying to import the library using relative path: ../../../ximc/crossplatform/wrappers/python ...")
    import sys
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    import libximc.highlevel as ximc
    print("Success!")


class SelectionManager:
    """Helps you to select the device to connect to"""
    class DeviceType(Enum):
        COM_DEVICE = 1
        VIRT_DEVICE = 2
        NETWORK_DEVICE = 3
        ALL_DEVICES = 4

    def get_uri(self) -> str:
        """Device selection Manager.

        :return: URI of the device to open
        :rtype: str
        """
        device_type = self.ask_for_device_type()
        if device_type == SelectionManager.DeviceType.COM_DEVICE:
            uri = self.ask_for_com_uri()
        elif device_type == SelectionManager.DeviceType.VIRT_DEVICE:
            uri = self.get_virt_uri()
        elif device_type == SelectionManager.DeviceType.NETWORK_DEVICE:
            uri = self.ask_for_network_uri()
        elif device_type == SelectionManager.DeviceType.ALL_DEVICES:
            devenum = self.get_devices_enumeration()

            if len(devenum) > 0:
                index = self.ask_for_device_index(devenum)
                uri = devenum[index]["uri"]
            else:
                # set path to virtual device file to be created
                tempdir = os.path.join(os.path.expanduser("~"), "testdevice.bin")
                uri = "xi-emu:///" + tempdir
        else:
            raise RuntimeError("Wrong device type! Got {}".format(device_type))
        return uri

    @interface_repeater
    def ask_for_device_type(self) -> DeviceType:
        print("What XIMC device do you want to open?\n"
              "Enter the key:\n"
              "1 - for COM device\n"
              "2 - for virtual device\n"
              "3 - network device\n"
              "4 - search for all available devices.\n")
        key = int(input("Your choice: "))
        return SelectionManager.DeviceType(key)

    def ask_for_com_uri(self) -> str:
        if platform.system() == "Windows":
            print(r"Enter the port number:")
            uri = r"xi-com:\\.\COM" + input(r"xi-com:\\.\COM")
        else:
            print("Enter the port number:")
            uri = "xi-com:/dev/tty.s" + input("xi-com:/dev/tty.s")
        return uri

    def get_virt_uri(self) -> str:
        return "xi-emu:///" + os.path.join(os.path.expanduser("~"), "virtual_controller.bin")

    def ask_for_network_uri(self) -> str:
        print("Enter the device's network address:")
        print("Example: 192.168.0.1/89ABCDEF, where 192.168.0.1 is an IP of the device and 89ABCDEF "
              "is a serial number.")
        return "xi-net://" + input()

    def get_devices_enumeration(self) -> 'list':
        # ******************************************** #
        #         Device searching and probing         #
        # ******************************************** #

        # Flags explanation:
        # ximc.EnumerateFlags.ENUMERATE_PROBE   -   Probing found devices for detailed info.
        # ximc.EnumerateFlags.ENUMERATE_NETWORK -   Check network devices.
        enum_flags = ximc.EnumerateFlags.ENUMERATE_PROBE | ximc.EnumerateFlags.ENUMERATE_NETWORK

        # Hint explanation:
        # "addr=" hint is used for broadcast network enumeration
        enum_hints = "addr="
        return ximc.enumerate_devices(enum_flags, enum_hints)

    @interface_repeater
    def ask_for_device_index(self, devenum: 'list[dict]') -> int:
        for i, dev in enumerate(devenum, 1):
            print("#{} - {}, SN{}: ".format(i, dev["uri"], dev["device_serial"]))
        print("Enter the device number:")
        index = int(input()) - 1
        if index >= len(devenum) or index < 0:
            raise ValueError("Device index is out of range!")
        return index


class GeneralManager:
    class Action(Enum):
        QUIT = "q"
        MOVE = "m"
        MOVE_CALB = "c"
        EXTIO = "i"
        CHANGE_SETTINGS = "s"

    def __init__(self, axis: ximc.Axis):
        self.axis = axis
        self.move_manager = MovementManager(axis)
        self.move_manage_calb = MovementManagerCalb(axis)
        self.extio_manager = EXTIOManager(axis)
        self.settings_manager = SettingsManager(axis)

        # Set default 1:1 scaling for *_calb commands
        self.axis.set_calb(1, self.axis.get_engine_settings().MicrostepMode)

    def start(self):
        action = self.ask_for_action()
        while action != GeneralManager.Action.QUIT:
            if action == GeneralManager.Action.MOVE:
                self.move_manager.start()
            if action == GeneralManager.Action.MOVE_CALB:
                self.move_manage_calb.start()
            elif action == GeneralManager.Action.EXTIO:
                self.extio_manager.start()
            elif action == GeneralManager.Action.CHANGE_SETTINGS:
                self.settings_manager.start()
            action = self.ask_for_action()
        print("Exiting initialized...")

    @interface_repeater
    def ask_for_action(self) -> Action:
        print("\n===== Main menu =====\n"
              "Choose action:\n"
              "Q or q keys\t-\texit\n"
              "M or m key\t-\tmove\n"
              "C or c key\t-\tmove (user-units)\n"
              "I or i keys\t-\texternal I/O EXTIO\n"
              "S or s keys\t-\tchange settings\n")
        key = input("Your choice: ")
        return GeneralManager.Action(key.lower())


axis: ximc.Axis = None


def main():
    """
    Starts Selection Manager and General Manager.
    """
    global axis

    print("Library version: " + ximc.ximc_version())

    uri = SelectionManager().get_uri()

    # ******************************************** #
    #              Create axis object              #
    # ******************************************** #
    # Axis is the main libximc.highlevel class. It allows you to interact with the device.
    # Axis takes one argument - URI of the device.
    axis = ximc.Axis(uri)
    print("\nOpen device " + axis.uri)
    axis.open_device()  # The connection must be opened manually

    GeneralManager(axis).start()

    print("\nClosing device connection... ", end="")
    axis.close_device()
    print("Done!")
    print("Exiting...")


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("Exiting initialized...")
        print("Powering off the device's windings... ", end="")
        axis.command_power_off()
        print("Done!")
        print("Exit program...")



import os
import re
import sys
from ctypes import *

if sys.version_info >= (3,0):
    import urllib.parse

try: 
    from libximc import *
except ImportError as err:
    cur_dir = os.path.abspath(os.path.dirname(__file__))
    ximc_dir = os.path.join(cur_dir, "..", "..", "..", "ximc")
    ximc_package_dir = os.path.join(ximc_dir, "crossplatform", "wrappers", "python")
    sys.path.append(ximc_package_dir)
    from libximc import *


def choose_device() -> bytes:
    devenum, device_count = search_devices()
    if device_count < 1:
        print("The real controller is not found or busy with another app.\nThe virtual controller is opening to check "
              "the operation of the library...")
        device_name = create_virtual_device_name()
    else:
        device_name = get_device_name_from_list(devenum, device_count)
    lib.free_enumerate_devices(devenum)
    return device_name


def create_virtual_device_name() -> bytes:
    path = os.path.expanduser("~") + "/virt_controller.bin"
    if os.altsep:
        path = path.replace(os.sep, os.altsep)
    # urlparse build wrong path if scheme is not file
    uri = urllib.parse.urlunparse(urllib.parse.ParseResult(scheme="file", netloc=None, path=path, params=None,
                                                           query=None, fragment=None))
    device_name = re.sub(r"^file", "xi-emu", uri).encode()
    print("Virtual controller file path: '{}'".format(path))
    return device_name


def get_device_name_from_list(devenum, device_count) -> bytes:
    while True:
        print_device_list(devenum, device_count)
        print("When using an Ethernet (TCP protocol) connection, multi-user interaction with the device is possible. "
              "In this case, the same device may be controlled simultaneously from multiple sources, which can "
              "potentially lead to control conflicts.\nEnter device number from the list:")
        try:
            number = int(input())
        except Exception:
            continue

        if 1 <= number <= device_count:
            device_name = lib.get_device_name(devenum, number - 1)
            print("Selected device: '{}'".format(device_name.decode("utf-8")))
            return device_name

        if number == device_count + 1:
            return create_virtual_device_name()


def print_device_list(devenum, device_count) -> None:
    print("\nFound devices list:")
    for i in range(device_count):
        raw_name = lib.get_device_name(devenum, i)
        device_name = raw_name.decode("utf-8")
        print(" #{}. {}".format(i + 1, device_name))
    print(" #{}. virtual".format(max(device_count, 0) + 1))


def search_devices():
    print("\nSearching for devices...")
    probe_flags = EnumerateFlags.ENUMERATE_PROBE | EnumerateFlags.ENUMERATE_NETWORK
    enumerate_hints = b"addr="
    devenum = lib.enumerate_devices(probe_flags, enumerate_hints)
    if not devenum:
        print("Error searching for devices")
        return devenum, 0

    device_count = lib.get_device_count(devenum)
    if device_count < 1:
        print("No devices found")
        return devenum, device_count

    print("Found {} devices".format(device_count + 1))
    return devenum, device_count


def warn_user_about_modifications() -> bool:
    while True:
        answer = input("\nThe script will modify all controller settings. Continue? [y/n]:\n")
        if answer.lower() == "y":
            return False

        if answer.lower() == "n":
            return True


def main() -> None:
    # A compilation of the profile file
    exec(compile(open("8MT173-25-MEn1.py").read(), "8MT173-25-MEn1", "exec"), globals())

    # variable 'lib' points to a loaded library
    # note that ximc uses stdcall on win
    sbuf = create_string_buffer(64)
    lib.ximc_version(sbuf)
    print("This is a ximc test program\nlibximc version " + sbuf.raw.decode().rstrip("\0"))

    if warn_user_about_modifications():
        return

    device_name = choose_device()
    print("\nOpening device...", end="")
    device = lib.open_device(device_name)
    if device == Result.NoDevice:
        print("\nError opening device\nPress Enter to exit...")
        return
    print(" Done")

    print("Setting profile for 8MT173-25...", end="")
    result = set_profile_8MT173_25_MEn1(lib, device)
    print(" Done. Result: {}".format(result))

    print("Closing device...", end="")
    # The device_t device parameter in this function is a C pointer, unlike most library functions that use this parameter
    lib.close_device(byref(cast(device, POINTER(c_int))))
    print(" Done")
    input("Press Enter to exit...")


if __name__ == "__main__":
    main()



def interface_repeater(func):
    """Supporting decorator. It allows repeat command request if enter invalid value."""
    def wrapper(*args, **kwargs):
        while True:
            try:
                return func(*args, **kwargs)
            except KeyboardInterrupt:
                raise KeyboardInterrupt("It's time to close!")
            except Exception:
                print("\nIncorrect input! Try again.")
    return wrapper



def set_profile_ST5909L3008(lib, id):
    worst_result = Result.Ok
    result = Result.Ok

    feedback_settings = feedback_settings_t()

    feedback_settings.IPS = 2000
    class FeedbackType_:
        FEEDBACK_ENCODER_MEDIATED = 6
        FEEDBACK_NONE = 5
        FEEDBACK_EMF = 4
        FEEDBACK_ENCODER = 1
    feedback_settings.FeedbackType = FeedbackType_.FEEDBACK_NONE
    class FeedbackFlags_:
        FEEDBACK_ENC_TYPE_BITS = 192
        FEEDBACK_ENC_TYPE_DIFFERENTIAL = 128
        FEEDBACK_ENC_TYPE_SINGLE_ENDED = 64
        FEEDBACK_ENC_FILTER_STRONG = 48
        FEEDBACK_ENC_FILTER_BITS = 48
        FEEDBACK_ENC_FILTER_MEDIUM = 32
        FEEDBACK_ENC_FILTER_WEAK = 16
        FEEDBACK_ENC_ADAPTIVE_HOLDING = 2
        FEEDBACK_ENC_REVERSE = 1
        FEEDBACK_ENC_FILTER_NONE = 0
        FEEDBACK_ENC_TYPE_AUTO = 0
    feedback_settings.FeedbackFlags = FeedbackFlags_.FEEDBACK_ENC_TYPE_SINGLE_ENDED | FeedbackFlags_.FEEDBACK_ENC_FILTER_NONE | FeedbackFlags_.FEEDBACK_ENC_TYPE_AUTO
    feedback_settings.CountsPerTurn = 2000
    result = lib.set_feedback_settings(id, byref(feedback_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    home_settings = home_settings_t()

    home_settings.FastHome = 100
    home_settings.uFastHome = 0
    home_settings.SlowHome = 10
    home_settings.uSlowHome = 0
    home_settings.HomeDelta = -1000
    home_settings.uHomeDelta = 0
    class HomeFlags_:
        HOME_USE_FAST = 256
        HOME_STOP_SECOND_BITS = 192
        HOME_STOP_SECOND_LIM = 192
        HOME_STOP_SECOND_SYN = 128
        HOME_STOP_SECOND_REV = 64
        HOME_STOP_FIRST_BITS = 48
        HOME_STOP_FIRST_LIM = 48
        HOME_STOP_FIRST_SYN = 32
        HOME_STOP_FIRST_REV = 16
        HOME_HALF_MV = 8
        HOME_MV_SEC_EN = 4
        HOME_DIR_SECOND = 2
        HOME_DIR_FIRST = 1
    home_settings.HomeFlags = HomeFlags_.HOME_USE_FAST | HomeFlags_.HOME_STOP_SECOND_REV | HomeFlags_.HOME_STOP_FIRST_BITS | HomeFlags_.HOME_DIR_FIRST
    result = lib.set_home_settings(id, byref(home_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    move_settings = move_settings_t()

    move_settings.Speed = 1000
    move_settings.uSpeed = 0
    move_settings.Accel = 2000
    move_settings.Decel = 4000
    move_settings.AntiplaySpeed = 1000
    move_settings.uAntiplaySpeed = 0
    class MoveFlags_:
        RPM_DIV_1000 = 1

    result = lib.set_move_settings(id, byref(move_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    engine_settings = engine_settings_t()

    engine_settings.NomVoltage = 0
    engine_settings.NomCurrent = 2120
    engine_settings.NomSpeed = 5000
    engine_settings.uNomSpeed = 0
    class EngineFlags_:
        ENGINE_USE_PEAK_CURRENT = 256
        ENGINE_LIMIT_RPM = 128
        ENGINE_LIMIT_CURR = 64
        ENGINE_LIMIT_VOLT = 32
        ENGINE_ACCEL_ON = 16
        ENGINE_ANTIPLAY = 8
        ENGINE_MAX_SPEED = 4
        ENGINE_CURRENT_AS_RMS = 2
        ENGINE_REVERSE = 1
    engine_settings.EngineFlags = EngineFlags_.ENGINE_LIMIT_RPM | EngineFlags_.ENGINE_ACCEL_ON
    engine_settings.Antiplay = 575
    class MicrostepMode_:
        MICROSTEP_MODE_FRAC_256 = 9
        MICROSTEP_MODE_FRAC_128 = 8
        MICROSTEP_MODE_FRAC_64 = 7
        MICROSTEP_MODE_FRAC_32 = 6
        MICROSTEP_MODE_FRAC_16 = 5
        MICROSTEP_MODE_FRAC_8 = 4
        MICROSTEP_MODE_FRAC_4 = 3
        MICROSTEP_MODE_FRAC_2 = 2
        MICROSTEP_MODE_FULL = 1
    engine_settings.MicrostepMode = MicrostepMode_.MICROSTEP_MODE_FRAC_256
    engine_settings.StepsPerRev = 400
    result = lib.set_engine_settings(id, byref(engine_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    entype_settings = entype_settings_t()

    class EngineType_:
        ENGINE_TYPE_BRUSHLESS = 5
        ENGINE_TYPE_TEST = 4
        ENGINE_TYPE_STEP = 3
        ENGINE_TYPE_2DC = 2
        ENGINE_TYPE_DC = 1
        ENGINE_TYPE_NONE = 0
    entype_settings.EngineType = EngineType_.ENGINE_TYPE_STEP | EngineType_.ENGINE_TYPE_NONE
    class DriverType_:
        DRIVER_TYPE_EXTERNAL = 3
        DRIVER_TYPE_INTEGRATE = 2
    entype_settings.DriverType = DriverType_.DRIVER_TYPE_INTEGRATE
    result = lib.set_entype_settings(id, byref(entype_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    power_settings = power_settings_t()

    power_settings.HoldCurrent = 50
    power_settings.CurrReductDelay = 1000
    power_settings.PowerOffDelay = 60
    power_settings.CurrentSetTime = 300
    class PowerFlags_:
        POWER_SMOOTH_CURRENT = 4
        POWER_OFF_ENABLED = 2
        POWER_REDUCT_ENABLED = 1
    power_settings.PowerFlags = PowerFlags_.POWER_SMOOTH_CURRENT | PowerFlags_.POWER_OFF_ENABLED | PowerFlags_.POWER_REDUCT_ENABLED
    result = lib.set_power_settings(id, byref(power_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    secure_settings = secure_settings_t()

    secure_settings.LowUpwrOff = 800
    secure_settings.CriticalIpwr = 4000
    secure_settings.CriticalUpwr = 5000
    secure_settings.CriticalT = 800
    secure_settings.CriticalIusb = 450
    secure_settings.CriticalUusb = 520
    secure_settings.MinimumUusb = 420
    class Flags_:
        ALARM_ENGINE_RESPONSE = 128
        ALARM_WINDING_MISMATCH = 64
        BRAKING_OVERVOLTAGE_PROTECTION = 32
        ALARM_FLAGS_STICKING = 16
        ALARM_ON_BORDERS_SWAP_MISSET = 8
        H_BRIDGE_ALERT = 4
        LOW_UPWR_PROTECTION = 2
        ALARM_ON_DRIVER_OVERHEATING = 1
    secure_settings.Flags = Flags_.ALARM_ENGINE_RESPONSE | Flags_.ALARM_FLAGS_STICKING | Flags_.ALARM_ON_BORDERS_SWAP_MISSET | Flags_.H_BRIDGE_ALERT | Flags_.ALARM_ON_DRIVER_OVERHEATING
    result = lib.set_secure_settings(id, byref(secure_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    edges_settings = edges_settings_t()

    class BorderFlags_:
        BORDERS_SWAP_MISSET_DETECTION = 8
        BORDER_STOP_RIGHT = 4
        BORDER_STOP_LEFT = 2
        BORDER_IS_ENCODER = 1

    class EnderFlags_:
        ENDER_SW2_ACTIVE_LOW = 4
        ENDER_SW1_ACTIVE_LOW = 2
        ENDER_SWAP = 1
    edges_settings.EnderFlags = EnderFlags_.ENDER_SW2_ACTIVE_LOW | EnderFlags_.ENDER_SW1_ACTIVE_LOW
    edges_settings.LeftBorder = -139400
    edges_settings.uLeftBorder = 0
    edges_settings.RightBorder = -2600
    edges_settings.uRightBorder = 0
    result = lib.set_edges_settings(id, byref(edges_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    pid_settings = pid_settings_t()

    pid_settings.KpU = 0
    pid_settings.KiU = 0
    pid_settings.KdU = 0
    pid_settings.Kpf = 0.0036
    pid_settings.Kif = 0.038
    pid_settings.Kdf = 2.8e-05
    result = lib.set_pid_settings(id, byref(pid_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    sync_in_settings = sync_in_settings_t()

    class SyncInFlags_:
        SYNCIN_GOTOPOSITION = 4
        SYNCIN_INVERT = 2
        SYNCIN_ENABLED = 1

    sync_in_settings.ClutterTime = 4
    sync_in_settings.Position = 0
    sync_in_settings.uPosition = 0
    sync_in_settings.Speed = 0
    sync_in_settings.uSpeed = 0
    result = lib.set_sync_in_settings(id, byref(sync_in_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    sync_out_settings = sync_out_settings_t()

    class SyncOutFlags_:
        SYNCOUT_ONPERIOD = 64
        SYNCOUT_ONSTOP = 32
        SYNCOUT_ONSTART = 16
        SYNCOUT_IN_STEPS = 8
        SYNCOUT_INVERT = 4
        SYNCOUT_STATE = 2
        SYNCOUT_ENABLED = 1
    sync_out_settings.SyncOutFlags = SyncOutFlags_.SYNCOUT_ONSTOP | SyncOutFlags_.SYNCOUT_ONSTART
    sync_out_settings.SyncOutPulseSteps = 100
    sync_out_settings.SyncOutPeriod = 2000
    sync_out_settings.Accuracy = 0
    sync_out_settings.uAccuracy = 0
    result = lib.set_sync_out_settings(id, byref(sync_out_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    extio_settings = extio_settings_t()

    class EXTIOSetupFlags_:
        EXTIO_SETUP_INVERT = 2
        EXTIO_SETUP_OUTPUT = 1
    extio_settings.EXTIOSetupFlags = EXTIOSetupFlags_.EXTIO_SETUP_OUTPUT
    class EXTIOModeFlags_:
        EXTIO_SETUP_MODE_OUT_BITS = 240
        EXTIO_SETUP_MODE_OUT_MOTOR_ON = 64
        EXTIO_SETUP_MODE_OUT_ALARM = 48
        EXTIO_SETUP_MODE_OUT_MOVING = 32
        EXTIO_SETUP_MODE_OUT_ON = 16
        EXTIO_SETUP_MODE_IN_BITS = 15
        EXTIO_SETUP_MODE_IN_ALARM = 5
        EXTIO_SETUP_MODE_IN_HOME = 4
        EXTIO_SETUP_MODE_IN_MOVR = 3
        EXTIO_SETUP_MODE_IN_PWOF = 2
        EXTIO_SETUP_MODE_IN_STOP = 1
        EXTIO_SETUP_MODE_IN_NOP = 0
        EXTIO_SETUP_MODE_OUT_OFF = 0
    extio_settings.EXTIOModeFlags = EXTIOModeFlags_.EXTIO_SETUP_MODE_IN_STOP | EXTIOModeFlags_.EXTIO_SETUP_MODE_IN_NOP | EXTIOModeFlags_.EXTIO_SETUP_MODE_OUT_OFF
    result = lib.set_extio_settings(id, byref(extio_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    brake_settings = brake_settings_t()

    brake_settings.t1 = 300
    brake_settings.t2 = 500
    brake_settings.t3 = 300
    brake_settings.t4 = 400
    class BrakeFlags_:
        BRAKE_ENG_PWROFF = 2
        BRAKE_ENABLED = 1
    brake_settings.BrakeFlags = BrakeFlags_.BRAKE_ENG_PWROFF
    result = lib.set_brake_settings(id, byref(brake_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    control_settings = control_settings_t()

    control_settings.MaxSpeed[0] = 100
    control_settings.MaxSpeed[1] = 1000
    control_settings.MaxSpeed[2] = 0
    control_settings.MaxSpeed[3] = 0
    control_settings.MaxSpeed[4] = 0
    control_settings.MaxSpeed[5] = 0
    control_settings.MaxSpeed[6] = 0
    control_settings.MaxSpeed[7] = 0
    control_settings.MaxSpeed[8] = 0
    control_settings.MaxSpeed[9] = 0
    control_settings.uMaxSpeed[0] = 0
    control_settings.uMaxSpeed[1] = 0
    control_settings.uMaxSpeed[2] = 0
    control_settings.uMaxSpeed[3] = 0
    control_settings.uMaxSpeed[4] = 0
    control_settings.uMaxSpeed[5] = 0
    control_settings.uMaxSpeed[6] = 0
    control_settings.uMaxSpeed[7] = 0
    control_settings.uMaxSpeed[8] = 0
    control_settings.uMaxSpeed[9] = 0
    control_settings.Timeout[0] = 1000
    control_settings.Timeout[1] = 0
    control_settings.Timeout[2] = 0
    control_settings.Timeout[3] = 0
    control_settings.Timeout[4] = 0
    control_settings.Timeout[5] = 0
    control_settings.Timeout[6] = 0
    control_settings.Timeout[7] = 0
    control_settings.Timeout[8] = 0
    control_settings.MaxClickTime = 300
    class Flags_:
        CONTROL_BTN_RIGHT_PUSHED_OPEN = 8
        CONTROL_BTN_LEFT_PUSHED_OPEN = 4
        CONTROL_MODE_BITS = 3
        CONTROL_MODE_LR = 2
        CONTROL_MODE_JOY = 1
        CONTROL_MODE_OFF = 0
    control_settings.Flags = Flags_.CONTROL_MODE_LR | Flags_.CONTROL_MODE_OFF
    control_settings.DeltaPosition = 1
    control_settings.uDeltaPosition = 0
    result = lib.set_control_settings(id, byref(control_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    joystick_settings = joystick_settings_t()

    joystick_settings.JoyLowEnd = 5
    joystick_settings.JoyCenter = 4350
    joystick_settings.JoyHighEnd = 8600
    joystick_settings.ExpFactor = 100
    joystick_settings.DeadZone = 50
    class JoyFlags_:
        JOY_REVERSE = 1

    result = lib.set_joystick_settings(id, byref(joystick_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    ctp_settings = ctp_settings_t()

    ctp_settings.CTPMinError = 3
    class CTPFlags_:
        CTP_ERROR_CORRECTION = 16
        REV_SENS_INV = 8
        CTP_ALARM_ON_ERROR = 4
        CTP_BASE = 2
        CTP_ENABLED = 1
    ctp_settings.CTPFlags = CTPFlags_.CTP_ERROR_CORRECTION | CTPFlags_.CTP_ENABLED
    result = lib.set_ctp_settings(id, byref(ctp_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    uart_settings = uart_settings_t()

    uart_settings.Speed = 115200
    class UARTSetupFlags_:
        UART_STOP_BIT = 8
        UART_PARITY_BIT_USE = 4
        UART_PARITY_BITS = 3
        UART_PARITY_BIT_MARK = 3
        UART_PARITY_BIT_SPACE = 2
        UART_PARITY_BIT_ODD = 1
        UART_PARITY_BIT_EVEN = 0
    uart_settings.UARTSetupFlags = UARTSetupFlags_.UART_PARITY_BIT_EVEN
    result = lib.set_uart_settings(id, byref(uart_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    network_settings = network_settings_t()

    network_settings.DHCPEnabled = 1
    network_settings.IPv4Address[0] = 255
    network_settings.IPv4Address[1] = 255
    network_settings.IPv4Address[2] = 255
    network_settings.IPv4Address[3] = 255
    network_settings.SubnetMask[0] = 0
    network_settings.SubnetMask[1] = 0
    network_settings.SubnetMask[2] = 0
    network_settings.SubnetMask[3] = 0
    network_settings.DefaultGateway[0] = 0
    network_settings.DefaultGateway[1] = 0
    network_settings.DefaultGateway[2] = 0
    network_settings.DefaultGateway[3] = 0
    result = lib.set_network_settings(id, byref(network_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    password_settings = password_settings_t()

    password_settings.UserPassword = bytes([48, 48, 48, 48, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_password_settings(id, byref(password_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    controller_name = controller_name_t()

    controller_name.ControllerName = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    class CtrlFlags_:
        EEPROM_PRECEDENCE = 1

    result = lib.set_controller_name(id, byref(controller_name))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    emf_settings = emf_settings_t()

    emf_settings.L = 0
    emf_settings.R = 0
    emf_settings.Km = 0
    class BackEMFFlags_:
        BACK_EMF_KM_AUTO = 4
        BACK_EMF_RESISTANCE_AUTO = 2
        BACK_EMF_INDUCTANCE_AUTO = 1
    emf_settings.BackEMFFlags = BackEMFFlags_.BACK_EMF_KM_AUTO | BackEMFFlags_.BACK_EMF_RESISTANCE_AUTO | BackEMFFlags_.BACK_EMF_INDUCTANCE_AUTO
    result = lib.set_emf_settings(id, byref(emf_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    engine_advanced_setup = engine_advanced_setup_t()

    engine_advanced_setup.stepcloseloop_Kw = 50
    engine_advanced_setup.stepcloseloop_Kp_low = 1000
    engine_advanced_setup.stepcloseloop_Kp_high = 33
    result = lib.set_engine_advanced_setup(id, byref(engine_advanced_setup))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    extended_settings = extended_settings_t()

    extended_settings.Param1 = 0
    result = lib.set_extended_settings(id, byref(extended_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    stage_name = stage_name_t()

    stage_name.PositionerName = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_stage_name(id, byref(stage_name))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    stage_information = stage_information_t()

    stage_information.Manufacturer = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    stage_information.PartNumber = bytes([83, 84, 53, 57, 48, 57, 76, 51, 48, 48, 56, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_stage_information(id, byref(stage_information))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    stage_settings = stage_settings_t()

    stage_settings.LeadScrewPitch = 1
    stage_settings.Units = bytes([100, 101, 103, 114, 101, 101, 0, 0])
    stage_settings.MaxSpeed = 0
    stage_settings.TravelRange = 360
    stage_settings.SupplyVoltageMin = 0
    stage_settings.SupplyVoltageMax = 0
    stage_settings.MaxCurrentConsumption = 0
    stage_settings.HorizontalLoadCapacity = 0
    stage_settings.VerticalLoadCapacity = 0
    result = lib.set_stage_settings(id, byref(stage_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    motor_information = motor_information_t()

    motor_information.Manufacturer = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    motor_information.PartNumber = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_motor_information(id, byref(motor_information))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    motor_settings = motor_settings_t()

    class MotorType_:
        MOTOR_TYPE_BLDC = 3
        MOTOR_TYPE_DC = 2
        MOTOR_TYPE_STEP = 1
        MOTOR_TYPE_UNKNOWN = 0
    motor_settings.MotorType = MotorType_.MOTOR_TYPE_UNKNOWN
    motor_settings.ReservedField = 0
    motor_settings.Poles = 0
    motor_settings.Phases = 0
    motor_settings.NominalVoltage = 0
    motor_settings.NominalCurrent = 0
    motor_settings.NominalSpeed = 0
    motor_settings.NominalTorque = 0
    motor_settings.NominalPower = 0
    motor_settings.WindingResistance = 0
    motor_settings.WindingInductance = 0
    motor_settings.RotorInertia = 0
    motor_settings.StallTorque = 0
    motor_settings.DetentTorque = 0
    motor_settings.TorqueConstant = 0
    motor_settings.SpeedConstant = 0
    motor_settings.SpeedTorqueGradient = 0
    motor_settings.MechanicalTimeConstant = 0
    motor_settings.MaxSpeed = 5000
    motor_settings.MaxCurrent = 0
    motor_settings.MaxCurrentTime = 0
    motor_settings.NoLoadCurrent = 0
    motor_settings.NoLoadSpeed = 0
    result = lib.set_motor_settings(id, byref(motor_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    encoder_information = encoder_information_t()

    encoder_information.Manufacturer = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    encoder_information.PartNumber = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_encoder_information(id, byref(encoder_information))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    encoder_settings = encoder_settings_t()

    encoder_settings.MaxOperatingFrequency = 100
    encoder_settings.SupplyVoltageMin = 0
    encoder_settings.SupplyVoltageMax = 0
    encoder_settings.MaxCurrentConsumption = 0
    encoder_settings.PPR = 500
    class EncoderSettings_:
        ENCSET_REVOLUTIONSENSOR_ACTIVE_HIGH = 256
        ENCSET_REVOLUTIONSENSOR_PRESENT = 64
        ENCSET_INDEXCHANNEL_PRESENT = 16
        ENCSET_PUSHPULL_OUTPUT = 4
        ENCSET_DIFFERENTIAL_OUTPUT = 1

    result = lib.set_encoder_settings(id, byref(encoder_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    hallsensor_information = hallsensor_information_t()

    hallsensor_information.Manufacturer = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    hallsensor_information.PartNumber = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_hallsensor_information(id, byref(hallsensor_information))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    hallsensor_settings = hallsensor_settings_t()

    hallsensor_settings.MaxOperatingFrequency = 0
    hallsensor_settings.SupplyVoltageMin = 0
    hallsensor_settings.SupplyVoltageMax = 0
    hallsensor_settings.MaxCurrentConsumption = 0
    hallsensor_settings.PPR = 0
    result = lib.set_hallsensor_settings(id, byref(hallsensor_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    gear_information = gear_information_t()

    gear_information.Manufacturer = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    gear_information.PartNumber = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    result = lib.set_gear_information(id, byref(gear_information))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    gear_settings = gear_settings_t()

    gear_settings.ReductionIn = 1
    gear_settings.ReductionOut = 1
    gear_settings.RatedInputTorque = 0
    gear_settings.RatedInputSpeed = 0
    gear_settings.MaxOutputBacklash = 0
    gear_settings.InputInertia = 0
    gear_settings.Efficiency = 0
    result = lib.set_gear_settings(id, byref(gear_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    accessories_settings = accessories_settings_t()

    accessories_settings.MagneticBrakeInfo = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    accessories_settings.MBRatedVoltage = 0
    accessories_settings.MBRatedCurrent = 0
    accessories_settings.MBTorque = 0
    class MBSettings_:
        MB_POWERED_HOLD = 2
        MB_AVAILABLE = 1

    accessories_settings.TemperatureSensorInfo = bytes([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    accessories_settings.TSMin = 0
    accessories_settings.TSMax = 0
    accessories_settings.TSGrad = 0
    class TSSettings_:
        TS_AVAILABLE = 8
        TS_TYPE_BITS = 7
        TS_TYPE_SEMICONDUCTOR = 2
        TS_TYPE_THERMOCOUPLE = 1
        TS_TYPE_UNKNOWN = 0
    accessories_settings.TSSettings = TSSettings_.TS_TYPE_THERMOCOUPLE | TSSettings_.TS_TYPE_UNKNOWN
    class LimitSwitchesSettings_:
        LS_SHORTED = 16
        LS_SW2_ACTIVE_LOW = 8
        LS_SW1_ACTIVE_LOW = 4
        LS_ON_SW2_AVAILABLE = 2
        LS_ON_SW1_AVAILABLE = 1

    result = lib.set_accessories_settings(id, byref(accessories_settings))

    if result != Result.Ok:
        if worst_result == Result.Ok or worst_result == Result.ValueError:
            worst_result = result

    return worst_result
